# -----------------------------------------------------------------------------
# (C) British Crown Copyright 2017-2021 Met Office.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.
#
# * Neither the name of the copyright holder nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

#--------------------------------------------------------------------
# Example script to demonstrate how to create ice mass budget ASCII files
# from standard monthly-mean SIMIP diaganostics
# including applying the mask, area weighting, and summation
#
# Ed Blockley, Feb 2019
#--------------------------------------------------------------------

import numpy as np
import glob
from netCDF4 import Dataset
import datetime as dt
import os

#--------------------------------------------------------------------
# Define and count input files
# EXAMPLE CODE ONLY: should be updated to reflect the filename format being used
fileGlob = 'input_files/*_1m_[0-9]*-[0-9]*.nc'
files = glob.glob(fileGlob)
flist = sorted(files)

# derive dates from file names
DATES = []
for ndate,file in enumerate(flist):
   # extract date from file name 
   # assuming file naming format cice_${jobID}i_1m_YYYYMMDD-YYYYMMDD.nc
   path, fname = os.path.split(file)
   fname = fname.split('_')[-1]
   DATES.append(dt.datetime(int(fname[0:4]), int(fname[4:6]), 15))

YEARS = [DATES[x].year for x in range(0,len(DATES))]
MONTHS = [DATES[x].month for x in range(0,len(DATES))]

#--------------------------------------------------------------------
# define sea ice budget variables
budgetVars =   [
                  'sidmassgrowthbot',
                  'sidmassgrowthwat',
                  'sidmassmelttop',
                  'sidmassmeltbot',
                  'sidmasslat',
                  'sidmasssi',
                  'sidmassevapsubl',
                  'sidmassdyn',
                  'total'   
               ]

#--------------------------------------------------------------------
# read in static fields: mask and grid-cell-areas
#
# mask
maskFile = 'arctic_region_mask_orca1.nc'
fh = Dataset(maskFile, mode='r')
# read in mask and trim from NEMO to CICE grid
# NB to not trim to CICE grid use fh.variables['region'][:,:]
mask = fh.variables['region'][1:-1,1:-1]  
fh.close()
#
# areas
fh = Dataset(flist[0], mode='r')
tarea = fh.variables['tarea'][:,:]  
fh.close()

#--------------------------------------------------------------------
# define output file and populate header
#
datafile='MetOffice_UKESM1N96ORCA1_HIST_001_ice.txt'
# define formats
title_format = "%1s %4s %6s %14s %12s %17s %17s %15s %15s %12s %12s %15s %12s %12s"
data_format = "%6i %6i %14.5e %12.5e %17.5e %17.5e %15.5e %15.5e %12.5e %12.5e %15.5e %12.5e %12.5e"
headers = ('#','Year','Month','Area (Km**2)', 'Mass (Kg)', 
                  'sidmassgrowthbot',    
                  'sidmassgrowthwat', 
                  'sidmassmelttop',    
                  'sidmassmeltbot',    
                  'sidmasslat',    
                  'sidmasssi',    
                  'sidmassevapsubl',   
                  'sidmassdyn',   
                  'total')

# create header
data_fileh = open(datafile,'w')
data_fileh.write('# Contact: Ann Keen ann.keen@metoffice.gov.uk')
data_fileh.write("\n")
data_fileh.write('# Corresponding HIST file: n/a')
data_fileh.write("\n")
data_fileh.write('# Components of the Arctic sea ice mass budget (Kg s-1):')
data_fileh.write("\n")
data_fileh.write(title_format % headers)
data_fileh.write("\n")

#--------------------------------------------------------------------
# loop over monthly files, calculate budgets, mass and area
# then output to ascii file
for ndate,file in enumerate(flist):
   #
   # reset/zero the budget for this month
   thisBudget = np.ma.masked_all([len(budgetVars)],dtype=float)
   #
   # open netcdf file 
   fh = Dataset(file, mode='r')
   #
   # calculate budget components
   for nvar,varName in enumerate(budgetVars[0:-1]):
      thisVar = fh.variables[varName][0,:,:]
      thisBudget[nvar] = np.sum(thisVar*tarea*mask,dtype=float)
   #
   # sum all for total
   thisBudget[-1] = np.sum(thisBudget[0:-1])
   #
   # calculate mass and area
   thisVar = fh.variables['simass'][0,:,:] 
   total_mass = np.sum(thisVar*tarea*mask,dtype=float)
   thisVar = fh.variables['siconc'][0,:,:]
   total_area = np.sum(thisVar*tarea*mask,dtype=float)
   total_area = total_area / 1e6    # convert from m^2 to km^2 
   #
   fh.close()
   #
   # add row to ascii file
   data_fileh.write(data_format % (DATES[ndate].year,
                    DATES[ndate].month,
                    total_area,
                    total_mass,
                    thisBudget[0],
                    thisBudget[1],
                    thisBudget[2],
                    thisBudget[3],
                    thisBudget[4],
                    thisBudget[5],
                    thisBudget[6],
                    thisBudget[7],
                    thisBudget[8]))
   data_fileh.write("\n")

# close output file
data_fileh.close()

